
//-----------------------------------------------------------
//
//	Softbolts Ltd. (c) all rights reserved
//
//	Warning: This  program is  protected by copyright law 
//		 and   international  treaties.  Unauthorised
//		 reproduction or distribution of this program,
//		 or  any portion  or it, may result in severe
//		 civil  and  criminal  penalties, and will be
//		 prosecuted to  the  maximum extent possible.
//
//              Author: A. E. Dawson, 14 Oct 2002
//              mailto: anthony@softbolts.com
//
//		Last Revised: 15 Oct 2002
//
//-----------------------------------------------------------

package com.softbolts.lib;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


/**
*
*	BaseServlet is the class used to derive all of the 
*	other servlet classes for connectors, transformers,
*	nodes, and monitors. Amongst other things it reads in 
*	the servlet.xml specification to find out what servlets
*	are running on the same node.
*
*	This is the base class use to encapsulate the descrition
*	of a servlet with its objects fields and methods.
*
*	@author Anthony E. Dawson anthony@softbolts.com
*
*/

//-----------------------------------------------------------
public abstract class BaseServlet implements Serializable
{
	private Hashtable si  	= null;                         /** Maps servlet typeName to the ServletSpec class si = servlet info.
								*/
	private ServletEnv se 	= null;                         /** Contains the servlet environment variables se = servlet environment.
								*/

								/** Default handler for SAX persing.
								*/
	public DefaultHandler handler = null;			
								/** The servlet root name as assigned by the servlet developer	
								    in the Servlet() contructor should be a unique name amongst 
								    the servlet name domain.
								 */
        public String	servletName = null;
                                                                /** Number of request received service + transactional
								 */
        public int	requestsReceived = 0;
                                                                /** The servlet type name is generally the same as the root name
								    however this is not guaranteed to remain so 
                                                                    is unique int the servlet type name domain.
								 */
        public SvrTyp	servletType = null;
								/** The host name on which the servlet is running.
								*/
        public String	servletHostName = null;
								/** The host name on which to find the engine.
								*/
        public String	engineHostName = null;
								/** This process name. Currently assigned at the command line.
								*/
        public String	procName = null;
								/** The instance name assigned by the engine when the servlet
								    registers itself.
								*/
        public String	assignedName = null;
								/** The port number for this servlet.
								*/
        public int	ourPort = 0;
								/** The port number on which to find the engine.
								*/
        public int	enginePort = 0;
								/** A map of configurable parameters for this servlet.
								*/
        public ConfigParamTyp		configParams = null;
								/** A map of Objects know by this servlet.
								*/
        public ServletObjTyp		objs = null;
								/** A list of transactions in which this this servlet is 
								    currently involved.
								*/
        public TransactionListTyp	transactionList = null;
								/** The run status for this servlet RunStatusTyp.halted 
								    || RunStatusTyp.running
								*/
	public RunStatusTyp		servletState = null;
								/** The mode in which requests are to be processed meaning spawn off 
								    a thread or not. ProcModeTyp.single_threaded || 
								    ProcModeTyp.multi_threaded
								*/
	public ProcModeTyp		procMode = null;
								/** The type of protocol to be used in communications
								    Protocol.xml || Protocol.http || Protocol.thunderbolt. Currently
								    only the Thunderbolt protocol is available.
								*/
        public ProtocolTyp		protocol = null;

								/** Abtract method which must be implemented in servlet.
								    This method is called when the servlet is instanciated
								    and every time servletState changed from RunStatusTyp.halted
								    to RunStatusTyp.running
								*/
        public abstract void init(NameValueTyp params) throws ThrowableString;
								/** Abtract method which must be implemented in servlet.
								    called once when the servlet shuts down.
								*/
        public abstract void finish(NameValueTyp params) throws ThrowableString;
								/** Abtract method which must be implemented in servlet.
								    called every before the servlet state is changed from 
								    RunStatusTyp.halted to RunStatusTyp.running.
								    The key sysService on the data buss accesses the
								    service requested.
								*/
        public abstract void service(NameValueTyp params) throws ThrowableString;
								/** Abtract method which must be implemented in servlet.
								    called at the start of a multi servlet transaction
								*/
        public abstract void beginTransaction(NameValueTyp params) throws ThrowableString;
								/** Abtract method which must be implemented in servlet.
								    called at the commit state of a multi servlet transaction
								*/
        public abstract void commitTransaction(NameValueTyp params) throws ThrowableString;
								/** Abtract method which must be implemented in servlet.
								    called in case of failure of a multi servlet transaction
								*/
        public abstract void rollBackTransaction(NameValueTyp params) throws ThrowableString;

	//-----------------------------------------------------------
	/**
		Initialises most of the Base sevlet variables to
		sensible values.
		Determins the local host name which will be used to 
		register this servlet with the engine.
		Reads in the servlet.xml specification.
	*/
	//-----------------------------------------------------------
	public BaseServlet()
	{
	 String hostName = "";
	 byte[] ipAddr;

		try {
			InetAddress addr = InetAddress.getLocalHost();

			// Get IP Address
			ipAddr = addr.getAddress();

			// Get hostname
			hostName = addr.getHostName();
		} catch (UnknownHostException e) {
			System.err.println("BaseServlet() - getHostName() - ");
		}

		servletHostName = hostName;
		servletType = SvrTyp.undefined;
		servletState = RunStatusTyp.running;
		protocol = ProtocolTyp.thunderbolt;
		procMode = ProcModeTyp.multi_threaded;
		ourPort = 0;
		System.err.println("Hostname returned: " + hostName);

		transactionList	= new TransactionListTyp();
		configParams	= new ConfigParamTyp();
		objs		= new ServletObjTyp();

									// read in the servlet specifications from the
									// servlets.xml file
		SAXParserFactory spf = SAXParserFactory.newInstance();
		spf.setValidating(false);
		spf.setNamespaceAware(true);
		String servletFn = "./config/servlets.xml";
		try {
									// parse the document
			SAXParser parser = spf.newSAXParser();
			handler = new XMLStreamHandler();
			InputSource source = new InputSource(new FileReader(servletFn));
			parser.parse(source, handler);

			si = ((XMLStreamHandler) handler).getServletSpec();
			se = ((XMLStreamHandler) handler).getServletEnv();

		} catch (Exception ex) {
			ex.printStackTrace(System.err);
			System.err.println("ERR: failed to read servlet configuration file: " + servletFn);
			System.exit(0);
		}

	}

	//-----------------------------------------------------------
/*
	//public synchronized Object clone()
	public Object clone()
	{
	 BaseServlet bsClone = null;

		try {
                        bsClone = (BaseServlet) ObjectCloner.deepCopy(this);
                                                        // these don't appear to be properly copied
                                                        // by the deep cloner.... search me why!
                                                        // if what is being assigned is a reference
                                                        // we are in trubble!
                        bsClone.servletState = this.servletState;
                        bsClone.servletType = this.servletType;
                        bsClone.procMode = this.procMode;
                        bsClone.protocol = this.protocol;

                } catch (Exception e) {
                        System.err.println(e);
                }
		return bsClone;
	}
*/
	//-----------------------------------------------------------
	/**
		Clears the obj map
	*/
	//-----------------------------------------------------------
	protected int ClearObjects()
	{
		objs.clear();
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Use to add an option to the configParams map.
	*/
	//-----------------------------------------------------------
	protected int AddOption(String paramName, String optName)
	{
		if (!configParams.containsKey(paramName))
		{
			System.err.println("ERR: AddOption() - paramName: " + paramName + " Not found in list.");
			return -1;
		}
		((ServletParamInfo) configParams.get(paramName)).opt.add(optName);
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Use to add an  object to the obj map.
	*/
	//-----------------------------------------------------------
	protected int AddObject(String objName)
	{
		objs.put(objName, new ServletObjectInfo());
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Use to set the servlet name to be use as the root
		name for all servlet instances of this type.
	*/
	//-----------------------------------------------------------
	protected int SetName(String servletName)
	{
		this.servletName = servletName;
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Use to add a field to an object. If the object
		does not exist AddObject is silently called.
	*/
	//-----------------------------------------------------------
	protected int AddField(String objName, String fieldName)
	{
		if (!objs.containsKey(objName))
		{
			AddObject(objName);
		}
								// get the object info from the obj name to obj info map
		ServletObjectInfo soi = (ServletObjectInfo) objs.get(objName); 
								// get the Obj fields map from the object info
								// add the new object to the obj field map
		soi.fields.put(fieldName, new ObjFieldInfo());
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Use to add a method to an object. Required minimum
		methods are Set and Get for each object you add.
	*/
	//-----------------------------------------------------------
	protected int AddMethod(String objName, String methodName, ServletObj obj)
	{
		if (!objs.containsKey(objName))
                {
                        AddObject(objName);
                }
		ServletObjectInfo soi = (ServletObjectInfo) objs.get(objName); 
		soi.methods.put(methodName, obj);
		return 0;
	}

	//-----------------------------------------------------------
	protected int AddConfigParam(String paramName, int size, String allow, boolean display)
	{
		ServletParamInfo spi = new ServletParamInfo();
		spi.size = size;
		spi.allow = allow;
		spi.display = display;
		spi.set = false;
		spi.value = "";

		configParams.put(paramName, spi);
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Checks to see if the particular parameter has been
		set.
	*/
	//-----------------------------------------------------------
	protected boolean ParamWasSet(String paramName)
	{
		return ((ServletParamInfo)configParams.get(paramName)).set;
	}

	//-----------------------------------------------------------
	/**
		Publish an even to the common channel. 
		This is not curently used but will in the future
		support publish and subsctibe.
		@param Is the transaction name.
	*/
	//-----------------------------------------------------------
	protected int PublishEvent(String transName)
	{
		return 0;
	}

	//-----------------------------------------------------------
	/**
		Publish an even to the common channel. 
		This is not curently used but will in the future
		support publish and subsctibe.
		@param the virtual data buss.
	*/
	//-----------------------------------------------------------
	protected int PublishEvent(NameValueTyp r)
	{
		return 0;
	}
}


